"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LibreOfficeUtils = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = tslib_1.__importDefault(require("path"));
const common_1 = require("../../common");
const constants_1 = require("./constants");
/**
 * Utility class for handling common tasks related to LibreOffice conversions.
 */
class LibreOfficeUtils {
    static getFileInfo(file) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (typeof file === 'string') {
                yield fs_1.promises.access(file, fs_1.constants.R_OK);
                const filename = path_1.default.basename(path_1.default.parse(file).base);
                return {
                    data: (0, fs_1.createReadStream)(file),
                    ext: path_1.default.extname(filename).slice(1)
                };
            }
            else {
                return { data: file.data, ext: file.ext };
            }
        });
    }
    /**
     * Adds files to the FormData object for LibreOffice conversion.
     *
     * @param {PathLikeOrReadStream[]} files - An array of files to be added to the FormData.
     * @param {FormData} data - The FormData object to which files will be added.
     * @throws {Error} Throws an error if the file extension is not supported.
     */
    static addFiles(files, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield Promise.all(files.map((file, key) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const fileInfo = yield this.getFileInfo(file);
                if (!constants_1.LIBRE_OFFICE_EXTENSIONS.includes(fileInfo.ext)) {
                    throw new Error(`${fileInfo.ext} is not supported`);
                }
                data.append('files', fileInfo.data, `${key}.${fileInfo.ext}`);
            })));
        });
    }
    /**
     * Adds page properties to the FormData object based on the provided PageProperties.
     *
     * @param {FormData} data - The FormData object to which page properties will be added.
     * @param {PageProperties} pageProperties - The page properties to be added to the FormData.
     */
    static addPageProperties(data, pageProperties) {
        if (pageProperties.landscape) {
            data.append('landscape', String(pageProperties.landscape));
        }
        if (pageProperties.nativePageRanges) {
            common_1.GotenbergUtils.assert(pageProperties.nativePageRanges.from > 0 &&
                pageProperties.nativePageRanges.to > 0 &&
                pageProperties.nativePageRanges.to >=
                    pageProperties.nativePageRanges.from, 'page ranges syntax error');
            data.append('nativePageRanges', `${pageProperties.nativePageRanges.from}-${pageProperties.nativePageRanges.to}`);
        }
        if (pageProperties.exportFormFields === false) {
            data.append('exportFormFields', String(pageProperties.exportFormFields));
        }
        if (pageProperties.singlePageSheets) {
            data.append('singlePageSheets', String(pageProperties.singlePageSheets));
        }
    }
    /**
     * Customizes the FormData object based on the provided conversion options.
     *
     * @param {FormData} data - The FormData object to be customized.
     * @param {ConversionOptions} options - The conversion options to apply to the FormData.
     * @returns {Promise<void>} A Promise that resolves once the customization is complete.
     */
    static customize(data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (options.pdfa) {
                data.append('pdfa', options.pdfa);
            }
            if (options.pdfUA) {
                data.append('pdfUA', String(options.pdfUA));
            }
            if (options.merge) {
                data.append('merge', String(options.merge));
            }
            if (options.metadata) {
                data.append('metadata', JSON.stringify(options.metadata));
            }
            if (options.properties) {
                LibreOfficeUtils.addPageProperties(data, options.properties);
            }
        });
    }
}
exports.LibreOfficeUtils = LibreOfficeUtils;
//# sourceMappingURL=libre-office.utils.js.map